from torch import nn

# from model.layer import WordRepresentation, FeedforwardLayer, BiaffineLayer
from transformers import AutoConfig
from vma_nlu.ner.pername_deeplearning.model.layer.wordnet import WordRepresentation
from vma_nlu.ner.pername_deeplearning.model.layer.FeedForward import FeedforwardLayer
from vma_nlu.ner.pername_deeplearning.model.layer.biaffine import BiaffineLayer

class Model(nn.Module):
    def __init__(self, args):
        super().__init__()

        config = AutoConfig.from_pretrained(args.model_name_or_path)
        
        self.use_pos = args.use_pos

        self.num_labels = args.num_labels
        
        self.lstm_input_size = args.num_layer_bert * config.hidden_size

        if args.use_char:
            self.lstm_input_size = self.lstm_input_size + 2 * args.char_hidden_dim
        
        if args.use_pos:
            self.lstm_input_size = self.lstm_input_size + args.feature_embed_dim

        if args.use_fasttext:
            self.lstm_input_size = self.lstm_input_size + 300
        
        self.word_rep = WordRepresentation(args)
        self.bilstm = nn.LSTM(input_size=self.lstm_input_size,
                            hidden_size=args.hidden_dim // 2,
                            num_layers=3,
                            bidirectional=True,
                            batch_first=True)
        self.feedStart = FeedforwardLayer(d_in=args.hidden_dim, d_hid=args.hidden_dim_ffw)
        self.feedEnd = FeedforwardLayer(d_in=args.hidden_dim, d_hid=args.hidden_dim_ffw)
        self.biaffine = BiaffineLayer(inSize1=args.hidden_dim, inSize2=args.hidden_dim, classSize=self.num_labels)

    def forward(self, input_ids=None, char_ids=None, fasttext_embs=None, first_subword=None, attention_mask=None, pos_ids=None):

        word_features = self.word_rep(input_ids=input_ids, 
                                    attention_mask=attention_mask,
                                    first_subword=first_subword,
                                    char_ids=char_ids,
                                    pos_ids=pos_ids)
        x, _ = self.bilstm(word_features)

        start = self.feedStart(x)
        end = self.feedEnd(x)

        score = self.biaffine(start, end)

        return score


from vma_nlu.ic.naive_bayes_classifier import NaiveBayesClassifier
from vma_nlu.ic.logistic_regression_classifier import LogisticRegressionClassifier
from vma_nlu.ner.pattern_extractor import PatternExtractor
from vma_nlu.ner.biaffine_extractor import BiaffineExtractor


import numpy as np
import pickle
import os
import re

WORK_DIR = os.path.dirname(os.path.abspath(__file__))
MODEL_PATH = os.path.join(WORK_DIR, "weight/LR_model.pkl")

model = pickle.load(open(MODEL_PATH, 'rb'))

def normalize_text(s):
    s = re.sub(r'([a-z])\1+', lambda m: m.group(1), s, flags=re.IGNORECASE)
    s = re.sub(r'([a-z][a-z])\1+', lambda m: m.group(1), s, flags=re.IGNORECASE)
    return s
text = ["hi", normalize_text("đặt lịchhhhh"), "hủy lịch", "đổi lịch", "hủy", "bỏ đi", "dừng lại", "không cần nữa", "làm lại"]
predict = model.predict(text)
confidence = np.max(model.predict_proba(text)[0])
print(predict)
print(confidence)



class Config(object):
    def __init__(self, label_set_path, char_vocab_path, checkpoint) -> None:
        super().__init__()

        self.model_name_or_path = 'vinai/phobert-base'
        self.pos_tag_set_path = None
        self.label_set_path = label_set_path
        self.char_vocab_path = char_vocab_path
        self.checkpoint = checkpoint

        self.use_pretrained = False

        # optional features
        self.use_pos = False
        self.use_char = True
        self.use_fasttext = False

        # Tuning Hyperparameters
        self.num_layer_bert = 4
        self.char_hidden_dim = 128
        self.char_embedding_dim = 100
        self.feature_embed_dim = 128
        self.hidden_dim = 728
        self.hidden_dim_ffw = 300
        self.char_vocab_size = 108
        self.pos_vocab_size = 23
        
        self.max_char_len = 20
        self.max_seq_len = 100

        self.num_labels = 2


import torch
from torch import nn
import numpy as np
from torch.nn import functional as F


class CharCNN(nn.Module):
    def __init__(self, hidden_dim, embedding_dim, vocab_size, dropout=0.1):
        super(CharCNN, self).__init__()
        self.hidden_dim = hidden_dim
        self.embeddings = nn.Embedding(vocab_size, embedding_dim)

        self.embeddings.weight.data.copy_ = torch.from_numpy(self.random_embedding(vocab_size, hidden_dim))

        self.kernels = [3, 4]
        cnns = []

        for k in self.kernels:
            seq = nn.Sequential(
                nn.Conv1d(embedding_dim, self.hidden_dim, kernel_size=k, padding=1),
                nn.Tanh()
            )
            cnns.append(seq)
        self.cnns = nn.ModuleList(cnns)
        self.dropout = nn.Dropout(dropout)

    def random_embedding(self, vocab_size, embedding_dim):
        pretrain_emb = np.empty([vocab_size, embedding_dim])
        scale = np.sqrt(3.0 / embedding_dim)
        for index in range(vocab_size):
            pretrain_emb[index, :] = np.random.uniform(-scale, scale, [1, embedding_dim])
        return pretrain_emb

    def forward(self, x):
        seq_len = x.size(1)
        batch_size = x.size(0)
        x = x.view(batch_size * seq_len, x.size(2))
        x = self.embeddings(x)
        x = torch.transpose(x, 2, 1)
        tmp = [cnn(x) for cnn in self.cnns]
        char_cnn = []
        for item in tmp:
            char_cnn.append(F.max_pool1d(item, item.size(2)))

        char_cnn = torch.cat(char_cnn, dim=1)
        char_out = char_cnn.view(batch_size, seq_len, -1)
        char_out = self.dropout(char_out)
        return char_out


import os
from setuptools import setup, find_packages

try:
    # pip >=20
    from pip._internal.network.session import PipSession
    from pip._internal.req import parse_requirements
except ImportError:
    try:
        # 10.0.0 <= pip <= 19.3.1
        from pip._internal.download import PipSession
        from pip._internal.req import parse_requirements
    except ImportError:
        # pip <= 9.0.3
        from pip.download import PipSession
        from pip.req import parse_requirements

long_description = ''

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements('requirements.txt', session=False)

# reqs is a list of requirement
# e.g. ['django==1.5.1', 'mezzanine==1.4.6']
try:
    reqs = [str(ir.req) for ir in install_reqs]
except:
    reqs = [str(ir.requirement) for ir in install_reqs]

VERSION = os.getenv('PACKAGE_VERSION', 'v0.0.1')[1:]

setup(
    name='vma_nlu',
    version='v1.2.7',
    description='VMA-NLU is a library to implement NLU component in virtual medical assistant.',
    long_description=long_description,
    include_package_data=True,
    url='https://github.com/demdecuong/NER_Extractor/tree/main',
    packages=find_packages(),
    author='minhnp',
    author_email='v.minhng@vinbrain.net',
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    install_requires=reqs,
    keywords='vma_nlu',
    python_requires='>=3.6',
    py_modules=['vma_nlu'],

)


import os
import pickle
import logging
import re
import numpy as np
from typing import Any, Dict, List, Optional, Text, Tuple, Type

from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn.linear_model import LogisticRegression, SGDClassifier
from sklearn.naive_bayes import MultinomialNB
from sklearn.pipeline import Pipeline
from sklearn.model_selection import train_test_split
from sklearn.svm import SVC

from rasa.nlu.components import Component
from rasa.nlu.classifiers.classifier import IntentClassifier
from rasa.shared.nlu.constants import INTENT, TEXT
from rasa.nlu.config import RasaNLUModelConfig
from rasa.shared.nlu.training_data.training_data import TrainingData
from rasa.shared.nlu.training_data.message import Message

WORK_DIR = os.path.dirname(os.path.abspath(__file__))
MODEL_PATH = os.path.join(WORK_DIR, "weight/LR_model.pkl")

logger = logging.getLogger(__name__)


class LogisticRegressionClassifier(IntentClassifier):
    name = "LogisticRegressionClassifier"

    def __init__(self, component_config: Optional[Dict[Text, Any]] = None, learner=None) -> None:
        super(LogisticRegressionClassifier, self).__init__(component_config)
        self.learner = learner

    def train(
        self,
        training_data: TrainingData,
        config: Optional[RasaNLUModelConfig] = None,
        **kwargs: Any,
    ) -> None:
        """Train LR classifier"""
        X_train = [e.get(TEXT) for e in training_data.intent_examples]
        y_train = [e.get(INTENT) for e in training_data.intent_examples]

        X_train, X_test, y_train, y_test = train_test_split(
            X_train, y_train, test_size=0.2, random_state=42)
        text_clf = Pipeline([('vect', CountVectorizer(ngram_range=(1, 2),
                                                      max_df=0.8,
                                                      max_features=None)),

                             ('clf', SGDClassifier(loss='log'))
                             ])
        self.learner = text_clf.fit(X_train, y_train)

    def process(self, message: Message, **kwargs: Any) -> None:
        """Process an incoming message."""

        if not self.learner:
            intent = None
            intent_ranking = []
        else:
            text = message.data.get('text')
            output = self.predict(text)
            intent = output.get("intent")
            intent_ranking = output.get("intent_ranking")

        message.set("intent", intent, add_to_output=True)
        message.set("intent_ranking", intent_ranking, add_to_output=True)

    def persist(self, file_name: Text, model_dir: Text) -> Optional[Dict[Text, Any]]:
        """Persist this component to disk for future loading."""
        classifier_file_name = file_name + "_classifier.pkl"
        pickle.dump(self.learner, open(os.path.join(model_dir, classifier_file_name), "wb"))

        return {"classifier": classifier_file_name}

    @classmethod
    def load(
        cls,
        meta: Dict[Text, Any],
        model_dir: Text,
        model_metadata: Optional["Metadata"] = None,
        cached_component: Optional["Component"] = None,
        **kwargs: Any,
    ) -> "Component":
        """Load this component from file."""
        classifier_file = os.path.join(model_dir, meta.get("classifier"))
        if not os.path.isfile(classifier_file):
            logger.error(
                f"File not found. Cannot load LR model: {classifier_file}")
            return cls(component_config=meta)
        else:
            try:
                learner = pickle.load(open(classifier_file, 'rb'))
                logger.debug(f"Loads LR model successfully ")
                return cls(meta, learner)
            except Exception as ex:
                logger.error(
                    f"Cannot load LR model: {classifier_file}: error: {ex}")

    def predict(self, text: Text):
        text = [self.normalize_text(text)]
        predict = self.learner.predict(text)
        confidence = np.max(self.learner.predict_proba(text)[0])
        return {
            "intent": {
                "name": predict[0],
                "confidence": confidence
            },
            "intent_ranking": []
        }

    def normalize_text(self, s):
        s = re.sub(r'([a-z])\1+', lambda m: m.group(1), s, flags=re.IGNORECASE)
        s = re.sub(r'([a-z][a-z])\1+', lambda m: m.group(1), s, flags=re.IGNORECASE)
        return s